# Evil Genius on the Oregon Trail 
# Copyright 2006 Jeff Armstrong (jba@member.fsf.org) 
# 
# This program is free software; you can redistribute it and/or modify 
# it under the terms of the GNU General Public License as published by 
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version. 
#  
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to:
# Free Software Foundation, Inc.
# 51 Franklin Street, Fifth Floor
# Boston, MA  02110-1301, USA

import random
import pygame
from pygame.locals import *
from gutil import *
import os.path

FINAL_SCALE = 5.0
MS_SCALE = 300
CHANGE_AFTER = 100
TOP_OFFSET = 40

class Character(pygame.sprite.Sprite):
    direction = 1
    image_list = None
    image_index = 0
    image_counter = 0
    grabbed = 0
    scaling = 0.0
    caught = 0
    annoyed = 0
    attack_or_capture_scale_time = 0
    attack_or_capture_scale_done = 0
    scale_image = None
    killed_player = 0

    # static properties
    name = None
    weight = 0
    attack_flag = 0
    speed = 1
    attack_image = None
    a_sound = None
    
    def __init__(self, static=0,filename=None,static_char=None,area=None):
        pygame.sprite.Sprite.__init__(self)

        if static: # This is a static character for storage, not for screen drawing

            # Load in the character attributes
            in_stream = open(os.path.join('data', filename),"r")
            self.name = in_stream.readline().strip()
            self.weight = int(in_stream.readline())
            self.attack_flag = int(in_stream.readline())
            self.speed = int(in_stream.readline())
            self.attack_image = load_image(in_stream.readline().strip())
            
            num_frames = int(in_stream.readline())
            self.image_list = []
            for i in range(0,num_frames):
                self.image_list.append(load_image(in_stream.readline().strip()))

            sound_name = in_stream.readline().strip()
            if sound_name != 'null':
                self.a_sound = pygame.mixer.Sound('data/'+sound_name)
            
            in_stream.close()


        else: # This is a player on the screen
            # Generate a y position
            y_pos = random.randint(area.top+TOP_OFFSET,area.bottom)

            # determine direction
            if random.randint(0,1):
                x_pos = area.left
                self.direction = 1
            else:
                x_pos = area.right
                self.direction = -1

            # Scale based on y immediately
            self.scaling = scale_on_field(y_pos)
            #self.image_list = static_char.get_image_list()
            self.image_list = []
            for pic in static_char.get_image_list():
                
                self.image_list = self.image_list + [pygame.transform.flip(pic,(self.direction==1),False)]

                # Scaling is bad...
                #new_width = int(self.scaling*static_char.get_image_list()[x].get_rect().width)
                #new_height = int(self.scaling*static_char.get_image_list()[x].get_rect().height)
                #pic = pygame.transform.scale(static_char.get_image_list()[x],(new_width,new_height)

            self.image = self.image_list[self.image_index]
            self.rect = self.image.get_rect()
            self.area = area

            # initialize locals
            self.speed = static_char.speed
            self.weight = static_char.weight
            self.attack_flag = static_char.attack_flag
            self.attack_image = static_char.attack_image
            self.attack_or_capture_scale_time = 0
            self.a_sound = static_char.a_sound

            self.rect.center = (x_pos,y_pos)

    def get_image_list(self):
        return self.image_list

    def off_screen(self):
        if self.direction == 1 and self.rect.left > self.area.right:
            return 1
        elif self.direction == -1 and self.rect.right < self.area.left:
            return 1
        return 0

    def set_grabbed(self):
        self.scale_image = self.image
        self.grabbed = 1

    def set_annoyed(self):
        self.scale_image = self.attack_image
        self.annoyed = 1

    def is_grabbed(self):
        return self.grabbed

    def is_caught(self):
        return self.caught

    def is_pissy(self):
        return self.attack_flag > 1

    def killed_you(self):
        return self.killed_player

    def get_sound(self):
        return self.a_sound

    def perform_scale(self):
        if self.attack_or_capture_scale_time == 0:
            self.attack_or_capture_scale_time = pygame.time.get_ticks()
        old_rect = self.rect
        scaling = 1.0 + float(pygame.time.get_ticks()-self.attack_or_capture_scale_time)/float(MS_SCALE)*(FINAL_SCALE-1.0)

        if scaling >= FINAL_SCALE:
            self.attack_or_capture_scale_done = 1
            scaling = FINAL_SCALE
        
        new_width = int(scaling*self.scale_image.get_rect().width)
        new_height = int(scaling*self.scale_image.get_rect().height)
        
        self.image = pygame.transform.scale(self.scale_image,(new_width,new_height))
        self.rect = self.image.get_rect()
        self.rect.center = old_rect.center
        
    
    def update(self):
        if self.grabbed:
            self.perform_scale()
            if self.attack_or_capture_scale_done:
                self.caught = 1
        elif self.annoyed and (self.attack_flag > 0):
            # Need to attack here
            self.perform_scale()
            if self.attack_or_capture_scale_done:
                self.killed_player = 1
        else:
            # Make it walk in the x direction
            self.rect.centerx = self.rect.centerx + self.direction*self.speed
            
            if pygame.time.get_ticks()-self.image_counter > CHANGE_AFTER:
                self.image_counter = pygame.time.get_ticks()
                self.image_index = self.image_index+1
                if self.image_index > len(self.image_list)-1:
                    self.image_index = 0
                self.image = self.image_list[self.image_index]
            

        
        
        
