# Evil Genius on the Oregon Trail 
# Copyright 2006 Jeff Armstrong (jba@member.fsf.org) 
# 
# This program is free software; you can redistribute it and/or modify 
# it under the terms of the GNU General Public License as published by 
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version. 
#  
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to:
# Free Software Foundation, Inc.
# 51 Franklin Street, Fifth Floor
# Boston, MA  02110-1301, USA

import random
import pygame
from pygame.locals import *
from gutil import *
from character import *

LEVEL_ANNOUNCE_LENGTH = 3000

class LevelAnnounce(pygame.sprite.Sprite):
    font_large = None
    font_small = None
    alpha = 128
    time_count = 0
    def __init__(self,screen,level):
        pygame.sprite.Sprite.__init__(self)
        self.image = pygame.Surface(screen.get_rect().size)
        #self.image.set_colorkey(self.image.get_at(1,1))
        self.image.convert()

        self.rect = self.image.get_rect()
        
        self.font_large = pygame.font.Font("data/underattack.ttf", 100)
        self.font_small = pygame.font.Font("data/underattack.ttf", 30)
        self.color =  Color('blue')
        render_level = self.font_large.render("Level "+str(level.number),1,self.color)
        render_name = self.font_small.render(level.name,1,self.color)
        render_goal = self.font_small.render("Goal Weight = "+str(level.goal),1,self.color)

        # draw the text onto the image - only need to do this once!
        self.image.fill(Color('white'))
        self.image.blit(render_level,(self.rect.centerx-render_level.get_rect().width/2,30))
        self.image.blit(render_name,(self.rect.centerx-render_name.get_rect().width/2,150))
        self.image.blit(render_goal,(self.rect.centerx-render_goal.get_rect().width/2,230))

        # A bit transparent
        self.image.set_alpha(self.alpha)

    def get_time_up(self):
        if self.time_count == 0:
            self.time_count = pygame.time.get_ticks()
            return 0
        else:
            return pygame.time.get_ticks() - self.time_count

    def is_done(self):
        if self.get_time_up() > LEVEL_ANNOUNCE_LENGTH:
            return 1
        else:
            return 0
    
    def update(self):
        # does nothing
        pass

class Level:
    characters = None
    probabilities = None
    ms_new_char = 1000
    name = None
    bg_image = None
    goal = 500
    number = 0

    last_time = 0
    
    def __init__(self,filename,number):

        self.number = number

        # Read the level properties
        in_stream = open(os.path.join('data', filename),"r")
        
        self.name = in_stream.readline().strip()
        num_chars = int(in_stream.readline().strip())
        self.characters = []
        for x in range(0,num_chars):
            self.characters.append(Character(static=1,filename=in_stream.readline().strip()+".txt"))

        probs = []
        for x in range(0,num_chars):
            probs.append(float(in_stream.readline().strip()))

        self.probabilities = [0.0]
        for x in range(1,num_chars):
            self.probabilities.append(self.probabilities[x-1]+probs[x-1])
            # print self.probabilities[x]

        self.goal = int(in_stream.readline().strip())
        self.bg_image = load_image(in_stream.readline().strip(),None)
        self.ms_new_char = int(in_stream.readline())

    def is_new_character(self):
        now_time = pygame.time.get_ticks()
        if now_time - self.last_time > self.ms_new_char:
            self.last_time = now_time
            return 1
        return 0

    def get_bg(self):
        return self.bg_image

    def get_name(self):
        return self.name

    def get_new_character(self,area):
        random_float = random.random()
        static_char = None
        for x in range(0,len(self.probabilities)):
            if random_float > self.probabilities[x]:
                static_char = self.characters[x]
                
        return Character(static_char=static_char,area=area)
            
