#!/usr/bin/env python

# Evil Genius on the Oregon Trail 
# Copyright 2006 Jeff Armstrong (jba@member.fsf.org) 
# 
# This program is free software; you can redistribute it and/or modify 
# it under the terms of the GNU General Public License as published by 
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version. 
#  
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to:
# Free Software Foundation, Inc.
# 51 Franklin Street, Fifth Floor
# Boston, MA  02110-1301, USA


import pygame
from pygame.locals import *
#import gutil
from gutil import *
from level import *
from character import *
from score import *
from menu import *
from final import *
from steam import *

# The fastest speed the sight can move
MAX_V = 20

# Time between registering antoher key operation
KEY_UPDATE_MS = 100

# Time between decels
TIME_DECEL_MS = 500

# TGrab speed multiplier
CLAW_MULTIPLIER = 5

# Pressure decrease/increase steps
P_DECREASE = 5
P_INCREASE = 1
P_MOVEMENT = 5
P_CLAW = 20

# Piston types
P_L_RIGHT = 1
P_S_RIGHT = 2
P_L_BOTTOM = 3
P_S_BOTTOM = 4

# Level data
NUMBER_LEVELS = 5
LEVEL_FILE_TEXT = "level"

# Display mode
DISP_MENU = 1
DISP_ANNOUNCE = 2
DISP_PLAY = 3
DISP_END = 4
DISP_HELP = 5
DISP_MANUAL = 6

class PressureBob(pygame.sprite.Sprite):
    y_pos = SCREENRECT.height-91
    x_empty = 402
    x_full = 580

    drawn_pressure = 100
    actual_pressure = 100
    length = 178
    def __init__(self):
        pygame.sprite.Sprite.__init__(self)
        self.image = load_image('steam_bob.png')
        self.rect = self.image.get_rect()
        self.rect.centery = self.y_pos
        self.rect.centerx = self.x_full
    def decrease(self,amount=5):
        self.actual_pressure = self.actual_pressure - amount
        if self.actual_pressure < 0:
            self.actual_pressure = 0
    def available(self,amount):
        if self.actual_pressure > amount:
            return 1
        else:
            return 0
    def update(self):
        if self.actual_pressure < self.drawn_pressure:
            self.drawn_pressure = self.drawn_pressure - P_DECREASE
        if self.actual_pressure > self.drawn_pressure:
            self.drawn_pressure = self.drawn_pressure + P_INCREASE
        if self.actual_pressure < 100:
            self.actual_pressure = self.actual_pressure + P_INCREASE

        # Calculate drawing
        self.rect.centerx = self.x_empty + (self.drawn_pressure*self.length)/100
        
class CPanel(pygame.sprite.Sprite):
    def __init__(self):
        pygame.sprite.Sprite.__init__(self)
        self.image = load_image('panel.png',None)
        self.rect = self.image.get_rect()
        screen = pygame.display.get_surface()
        self.rect.bottomleft = screen.get_rect().bottomleft
    def update(self):
        pass

class PistonBar(pygame.sprite.Sprite):
    holder = None
    p_type = 0
    def __init__(self, holder, sight, p_type, screen):
        pygame.sprite.Sprite.__init__(self)
        self.p_type = p_type

        # Load the appropriate image
        if self.p_type == P_L_RIGHT:
            self.image = load_image('horiz_bar_large.png')
        elif self.p_type == P_S_RIGHT:
            self.image = load_image('horiz_bar_small.png')
        elif self.p_type == P_L_BOTTOM:
            self.image = load_image('vert_bar_large.png')
        elif self.p_type == P_S_BOTTOM:
            self.image = load_image('vert_bar_small.png')
        else:
            print "No Image Found"

        self.rect = self.image.get_rect()
        self.holder = holder
        #self.area = sight.area
        self.area = screen.get_rect()

    def update(self):
        position = self.rect

        if self.p_type == P_L_RIGHT:
            position.centery = self.holder.rect.centery
            position.right = self.area.right
            position.width = (self.area.right-self.holder.rect.right)/2
        elif self.p_type == P_S_RIGHT:
            position.centery = self.holder.rect.centery
            position.left = self.holder.rect.right
            position.width = (self.area.right-self.holder.rect.right)/2+3
        elif self.p_type == P_L_BOTTOM:
            position.centerx = self.holder.rect.centerx
            position.bottom = self.area.bottom
            position.height = (self.area.bottom-self.holder.rect.bottom)/2
        elif self.p_type == P_S_BOTTOM:
            position.centerx = self.holder.rect.centerx
            position.top = self.holder.rect.bottom-15
            position.height = (self.area.bottom-self.holder.rect.bottom)/2+18
        if position.width<1:
            position.width=1
        if position.height<1:
            position.height=1
        
        self.rect = position
        self.image = pygame.transform.scale(self.image,(position.width,position.height))
    

class Holder(pygame.sprite.Sprite):
    sight = None
    def __init__(self, sight):
        pygame.sprite.Sprite.__init__(self)
        self.image = load_image('claw_holder.png')
        self.rect = self.image.get_rect()

        # Store the sight object
        self.sight = sight
        self.rect.midtop = self.sight.rect.midbottom

    def update(self):
        self.rect.midtop = self.sight.rect.midbottom
        if self.rect.bottom < self.sight.area.bottom:
            self.image.set_clip(clip_bottom(self.rect,self.sight.area))
        else:
            self.image.set_clip(None)

class Claw(pygame.sprite.Sprite):
    sight = None
    holder = None
    image_open = None
    image_closed = None

    # For handling grabbing
    grabbing = 0
    grab_position = 0
    grab_direction = 1
    grab_length = 0
    grab_time = 0

    grab_final = 0

    base_width = 0
    base_height = 0
    
    def __init__(self, sight, holder):
        pygame.sprite.Sprite.__init__(self)
        self.image_open = load_image('claw_open.png')
        self.image_closed = load_image('claw_closed.png')
        self.image = self.image_open
        self.rect = self.image.get_rect()
        
        self.sight = sight
        self.holder = holder
        
        # Grab length
        self.grab_length = self.holder.rect.centery - self.sight.rect.centery
        
        self.base_width = self.rect.width
        self.base_height = self.rect.height

    def grab(self):
        if not self.grabbing:
            self.grabbing = 1

    def grab_now(self):
        return self.grab_final

    def grab_update(self):
        if not self.grabbing:
            return
        
        self.rect = self.rect.move(0,-1*self.grab_position)
        self.grab_position = self.grab_position + CLAW_MULTIPLIER*self.grab_direction

        scale_target = scale_on_field(self.rect.centery)

        if(self.grab_position >= self.grab_length):
            self.grab_direction = -1
            self.grab_final = 1
        else:
            self.grab_final = 0

        image_of_choice = self.image_open
        if self.grab_direction < 0:
            image_of_choice = self.image_closed
        
        ratio = float(self.grab_position)/float(self.grab_length)
        scaling = 1.0-(1.0-scale_target)*ratio

        new_width = int(scaling*self.base_width)
        new_height = int(scaling*self.base_height)
        
        self.image = pygame.transform.scale(image_of_choice,(new_width,new_height))
        old_rect = self.rect
        self.rect = self.image.get_rect(center=old_rect.center)
        
        if(self.grab_position <= 0 and self.grab_direction == -1):
            self.grabbing = 0
            self.grab_direction = 1
            self.image = self.image_open
                

    def update(self):
        self.rect.center = self.holder.rect.center 
        if self.grabbing:
            self.grab_update()
        

class Sight(pygame.sprite.Sprite):
    v_horiz = 0
    v_vert = 0
    d_time = 0
    def __init__(self):
        pygame.sprite.Sprite.__init__(self)
        self.image = load_image('sight.png')
        self.rect = self.image.get_rect()
        
        screen = pygame.display.get_surface()
        self.area = Rect(0,0,screen.get_rect().width,screen.get_rect().height-STATUS_RECT.height)
        self.rect.center = screen.get_rect().center;
    
    def move_up(self):
        self.v_vert = self.v_vert - 1
        if self.v_vert < -1*MAX_V:
            self.v_vert = -1*MAX_V

    def move_down(self):
        self.v_vert = self.v_vert + 1
        if self.v_vert > MAX_V:
            self.v_vert = MAX_V
    
    def move_left(self):
        self.v_horiz = self.v_horiz - 1
        if self.v_horiz < -1*MAX_V:
            self.v_horiz = -1*MAX_V
            
    def move_right(self):
        self.v_horiz = self.v_horiz + 1
        if self.v_horiz > MAX_V:
            self.v_horiz = MAX_V

    def pre_update(self):
        self.rect = self.rect.move(self.v_horiz,self.v_vert)

        # Deceleration
        if pygame.time.get_ticks() - self.d_time > TIME_DECEL_MS:
            if self.v_horiz > 0:
                self.v_horiz = self.v_horiz-1
            elif self.v_horiz < 0:
                self.v_horiz = self.v_horiz+1

            if self.v_vert > 0:
                self.v_vert = self.v_vert-1
            elif self.v_vert < 0:
                self.v_vert = self.v_vert+1
            self.d_time = pygame.time.get_ticks()

        # Clipping
        if self.rect.left < self.area.left:
            self.rect.left = self.area.left
            self.v_horiz = -1*self.v_horiz
        elif self.rect.right > self.area.right:
            self.rect.right = self.area.right
            self.v_horiz = -1*self.v_horiz

        if self.rect.top < self.area.top:
            self.rect.top = self.area.top
            self.v_vert = -1*self.v_vert
        elif self.rect.bottom > self.area.bottom:
            self.rect.bottom > self.area.bottom
            self.v_vert = -1*self.v_vert
            
    def update(self):
        pass

def main():

    # Debug mode
    debug = 0
    
    # Initialize pygame
    sound = 1
    pygame.init()
    if pygame.mixer and not pygame.mixer.get_init():
        print 'Warning, no sound'
        pygame.mixer = None
    else:
        pygame.mixer.init()

    # Set the display mode
    screen = pygame.display.set_mode(SCREENRECT.size)
    
    #decorate the game window
    #icon = pygame.transform.scale(img, (32, 32))
    #pygame.display.set_icon(icon)
    pygame.display.set_caption(TITLE)
    pygame.mouse.set_visible(0)

    # Load sound effects
    if sound:
        s_steam = pygame.mixer.Sound('data/steam.wav')
        s_grab = pygame.mixer.Sound('data/clang.wav')
        try:
            pygame.mixer.music.load('data/phi.mid')
            pygame.mixer.music.play(-1)
        except:
            print "MIDI is not working..."
            print "Disabling music."

    # Background init
    # Should load images in the future
    background = pygame.Surface(screen.get_size())
    background = background.convert()
    background.fill((250, 250, 250))
    
    clock = pygame.time.Clock()

    # Sprite initialization

    # Player sprites
    sight = Sight()
    holder = Holder(sight)
    claw = Claw(sight,holder)
    control_panel = CPanel()
    pressure = PressureBob()
    score = Score()

    # Bars
    bottom_l = PistonBar(holder,sight,P_L_BOTTOM,screen)
    bottom_s = PistonBar(holder,sight,P_S_BOTTOM,screen)
    right_l = PistonBar(holder,sight,P_L_RIGHT,screen)
    right_s = PistonBar(holder,sight,P_S_RIGHT,screen)
    
    playersprites = pygame.sprite.RenderUpdates([sight,claw])
    pistonsprites = pygame.sprite.RenderUpdates([bottom_s,bottom_l,right_s,right_l,holder])
    panelsprites = pygame.sprite.RenderUpdates([control_panel,pressure,score])
    charsprites = pygame.sprite.RenderUpdates()
    messagesprites = pygame.sprite.RenderUpdates()
    steamsprites = pygame.sprite.RenderUpdates()

    # Level handling
    level_count = 0
    next_level = 0
    level_announce = None
    level = None

    # Trigger menu at the start
    trigger_menu = 1
    menu_sprite = MainMenu(screen)

    # triggers
    trigger_dead = 0
    trigger_win = 0
    trigger_lose = 0
    
    display_mode = DISP_PLAY

    # main loop
    holding = None
    time_count = 0
    steam_time_count = 0
    while 1:
        clock.tick(60)
        current_time = pygame.time.get_ticks()

        if next_level:
            menu_sprite.set_resumegame()
            level_count = level_count + 1
            if level_count<=NUMBER_LEVELS:
                level = Level(LEVEL_FILE_TEXT+str(level_count)+".txt",level_count)
                level_announce = LevelAnnounce(screen,level)
                background = level.get_bg()
                screen.blit(background,(0,0))
                pygame.display.flip()
                messagesprites.add(level_announce)
                display_mode = DISP_ANNOUNCE
                score.reset()
            else:
                trigger_win = 1
            next_level = 0

        # Display the menu
        if trigger_menu:
            messagesprites.add(menu_sprite)
            trigger_menu = 0
            display_mode = DISP_MENU

        if trigger_win:
            messagesprites.add(Final(MESSAGE_WIN))
            display_mode = DISP_END
            trigger_win = 0

        if trigger_lose:
            messagesprites.add(Final(MESSAGE_LOSE))
            display_mode = DISP_END
            trigger_lose = 0

        # Regular Play mode
        if display_mode == DISP_PLAY:
            for event in pygame.event.get():
                if event.type == QUIT:
                        return
                elif event.type == KEYDOWN:
                    if event.key == K_SPACE and pressure.available(P_CLAW):
                        claw.grab()
                        pressure.decrease(P_CLAW)
                        s_grab.play()
                    elif event.key == K_ESCAPE:
                        trigger_menu = 1
                    elif debug and event.key == K_TAB:
                        next_level = 1
                    else:
                        if sound:
                            s_steam.play(-1,2000)
                        holding = event.key
                elif event.type == KEYUP:
                    holding = None
                    
                    
            if holding != None:
                if current_time - time_count > KEY_UPDATE_MS and pressure.available(P_MOVEMENT):
                    if holding == K_LEFT:
                        sight.move_left()
                    elif holding == K_RIGHT:
                        sight.move_right()
                    elif holding == K_UP:
                        sight.move_up()
                    elif holding == K_DOWN:
                        sight.move_down()
                    time_count = current_time
                    pressure.decrease(P_MOVEMENT)
                    if sound:
                        s_steam.fadeout(2000)

            # Time to check for a capture
            if claw.grab_now():
                for prey in pygame.sprite.spritecollide(claw,charsprites,False):
                    if pressure.available(required_pressure(prey.weight)):
                        prey.set_grabbed()
                        sound = prey.get_sound()
                        if sound != None and sound:
                            sound.play()
                    else:
                        prey.set_annoyed()
                    pressure.decrease(required_pressure(prey.weight))
                
            for chars in charsprites.sprites():

                # Clean up if necessary
                if chars.is_caught():
                    score.increment_score(chars.weight)
                    charsprites.remove(chars)
                elif chars.off_screen() and (not chars.is_grabbed()):
                    charsprites.remove(chars)
                    
                # For tempermental characters
                if claw.grab_now() and chars.is_pissy():
                    chars.set_annoyed()

                # Check for kills
                if chars.killed_you():
                    trigger_lose = 1

            # Check for goal achieved
            if score.get_score() > level.goal:
                next_level = 1

            # Make steam come out of pistons
            if current_time - steam_time_count > NEW_STEAM_INTERVAL_MS:
                steam_time_count = current_time
                if random.randint(0,1):
                    steamsprites.add(Steam(right_s.rect.midright))
                else:
                    steamsprites.add(Steam(bottom_s.rect.midbottom))

            # Check for killing off steam
            for steam in steamsprites.sprites():
                if steam.is_done():
                    steamsprites.remove(steam)


        # Announcing Something            
        elif display_mode == DISP_ANNOUNCE:
            if level_announce.is_done():
                messagesprites.remove(level_announce)
                level_announce = None
                display_mode = DISP_PLAY

        # Menu
        elif display_mode == DISP_MENU:
            option_selected = 0
            for event in pygame.event.get():
                if event.type == QUIT:
                        return
                elif event.type == KEYDOWN:
                    if event.key == K_SPACE or event.key == K_RETURN:
                        option_selected = 1
                    elif event.key == K_ESCAPE:
                        menu_sprite.set_selection(MENU_QUIT)
                        option_selected = 1
                    elif event.key == K_UP:
                        menu_sprite.decrement_selected()
                    elif event.key == K_DOWN:
                        menu_sprite.increment_selected()

            if option_selected:
                if menu_sprite.get_selection() == MENU_QUIT:
                    return
                elif menu_sprite.get_selection() == MENU_HELP:
                    display_mode = DISP_MANUAL
                    messagesprites.remove(menu_sprite)
                    messagesprites.add(Final(MESSAGE_HELP))
                else:
                    # Close the menu and begin play
                    if menu_sprite.is_newgame():
                        next_level = 1
                    else:
                        next_level = 0
                        display_mode = DISP_PLAY
                    messagesprites.remove(menu_sprite)
                    
        elif display_mode == DISP_END:
            for event in pygame.event.get():
                if event.type == QUIT:
                        return
                elif event.type == KEYDOWN:
                    if event.key == K_SPACE or event.key == K_ESCAPE:

                        # Some serious cleanup
                        trigger_win = 0
                        trigger_lose = 0
                        level_count = 0
                        next_level = 0
                        menu_sprite.set_newgame()

                        # Clear all the character sprites
                        charsprites.empty()

                        # remove the message and trigger the menu
                        messagesprites.empty()
                        trigger_menu = 1
                        
        elif display_mode == DISP_MANUAL:
            for event in pygame.event.get():
                if event.type == QUIT:
                        return
                elif event.type == KEYDOWN:
                    if event.key == K_SPACE or event.key == K_ESCAPE:
                        trigger_menu=1
                        messagesprites.empty()

        # Add a new character to the screen
        if level != None:
            if level.is_new_character():
                charsprites.add(level.get_new_character(sight.area))

        charsprites.clear(screen,background)
        playersprites.clear(screen,background)
        panelsprites.clear(screen,background)
        messagesprites.clear(screen,background)
        steamsprites.clear(screen,background)
        pistonsprites.clear(screen,background)

        # Precalculates the position
        sight.pre_update()
        
        playersprites.update()
        pistonsprites.update()
        charsprites.update()
        panelsprites.update()
        messagesprites.update()
        steamsprites.update()

        dirty = charsprites.draw(screen) + \
                playersprites.draw(screen) + \
                pistonsprites.draw(screen) + \
                steamsprites.draw(screen) + \
                panelsprites.draw(screen) + \
                messagesprites.draw(screen)
                
        pygame.display.update(dirty)


#call the "main" function if running this script
if __name__ == '__main__': main()
